

/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.001        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: cchk.c
 * Program name: cchk.c
 * Source of file: PD SIG library.
 * Purpose: A simple C source checker 
 * Changes: Modified by Martin Houston to use file IO library instead
 *          of primitive read. Also will only produce verbose output
 *	    if the VERBOSE flag is defined.
 ********************************************************************/

/****************************************************************
*                                                               *
*                       CC (C Checker)                          *
*                                                               *
*            C Source Paren, bracket and comment Checker        *
*                                                               *
*                T. Jennings  -- Sometime in 1983               *
*                                                               *
*                                                               *
****************************************************************/

#include <stdio.h>
#include <ctype.h>

/* Very crude but very effective C source debugger. Counts the numbers of
matching braces, parenthesis and comments, and displays them at the left edge
of the screen. The best way to see what it does is to do it; try

        CCHK CCHK.C          C Check CCHK.C


Properly handles parens and brackets inside comments; they are ignored.
*/

main(argc,argv)
int argc;
char **argv;
{
FILE *file;
char *iobuf;	/* we will use a setbuf to allocate a decent amount
		 * of input buffering. */
char *badc;
int c,lastc;
int parens,brackets,comments;
int line, col;

        if ((argc < 2) || ((file = fopen(argv[1],"r")) == NULL))
	{
                printf("File missing. Try CC <filename.ext> \n");
                exit();
        }
	/* 10k buffer seems reasonable */
	if((iobuf = malloc(BUFSIZ * 10)) == NULL)
	{
                printf("Cannot get memory for input buffer!\n");
                exit();
        }
	setbuf(file, iobuf);

        brackets= parens= comments= 0;
        line= 0; col= 0;
        lastc= '\0';
	badc = NULL;

        while ((c = getc(file)) != EOF) 
	{

                if (col == 0)
		{
#ifdef VERBOSE
                        printf("%4d: {%4d} (%4d) /*%4d*/\n",
				line,brackets,parens,comments);
#endif
                }

/* Dont count parens and brackets that are inside comments. This of course
assumes that comments are properly matched; in any case, that will be the
first thing to look for. */

                if (!comments) 
		{
                        if (c == '{') ++brackets;
                        if (c == '(') ++parens;
                        if (c == '}') --brackets;
                        if (c == ')') --parens;
                }

/* Now do comments. This properly handles nested comments, whether or
not the compiler does is your responsibility */

                if ((c == '*') && (lastc == '/')) ++comments;
                if ((c == '/') && (lastc == '*')) --comments;

		if(brackets < 0)
		{
			badc = "bracket";
			brackets = 0;
		}
		if(parens < 0)
		{
			badc = "paranthesis";
			parens = 0;
		}
		if(comments < 0)
		{
			badc = "comment";
			comments = 0;
		}
		if(badc)
		{
			printf("Unbalanced %s on line %d col %d\n", 
				badc,line, col);
			badc = NULL;
		}
                ++col;
                if (c == '\n') 
		{                /* newline == New Line */
                        col= 0;                 /* set column 0 */
                        ++line;
                }
#ifdef VERBOSE
                putchar(c);                     /* display text */
#endif
                lastc= c;                       /* update last char */
        }
#ifdef VERBOSE
        printf("\n\n");
#endif
        if (brackets) printf("Unbalanced brackets\n");
        if (parens) printf("Unbalanced parenthesis\n");
        if (comments) printf("Unbalanced comments\n");
}
