
/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.001        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: hextobin.c
 * Program name: hextobin
 * Source of file: Ron Wellstead.
 * Purpose: converts a hex dump listing into a binary file.
 * Changes: <who what when & why major changes have been made>      
 ********************************************************************/

/*
 *
 *	@(#) hexbin.c 1.1 87/07/27 
 *
 */
#include <stdio.h>
#include <ctype.h>

char what[]="@(#) hexbin VR 1.0.0 4 Jun 86";

#define	byte unsigned char
#define NEW

FILE		*inf, *outf;
byte		checksum;
unsigned	MIN_SEG = 0xFFFF;
unsigned	MAX_SEG = 0;
unsigned	segment;
#ifdef	NEW
unsigned	length;
#endif

main (argc, argv)
int argc;
char *argv[];
{
	unsigned	i;

	if (argc != 3)
	{
		printf ("\n\nUseage:	HEX2BIN infile outfile\n\n");
		exit ();
	}

	inf = fopen (*++argv, "rb");
	if (inf == NULL)
	{
		printf ("Unable to open %s\n", *argv);
		exit ();
	}
	outf = fopen (*++argv, "wb");
	if (outf == NULL)
	{
		printf ("Unable to open %s\n", *argv);
		exit ();
	}

	MINSEG ();
	printf ("Minimum segment %04X\n",MIN_SEG);
	printf ("Maximum segment %04X\n",MAX_SEG);

	fseek (inf, 0L, 0);		/* Seek to begining of file */
	fseek (outf, 0L, 0);		/* Seek to begining of file */

	HEX2BIN ();
}

HEX2BIN ()
{
	length = 0;		/* Initial file length */

	while (1)
	{
		if (SCAN_RECORD () == 1)
		{
			printf ("Conversion complete\n");
			exit ();
		}
	}
}

MINSEG ()
{
	while (1)
	{
		if (SCAN_SEG () == 1)
		{
			break;
		}
	}
}

SCAN_COLON ()
{
	char	c;

	checksum = 0;

	while (1)
	{
		c = getc (inf);
		if (c == ':')
			return;
		if (c == EOF)
			ERR_EXIT ("Unexpected end of file");
	}
}

byte SCAN1HEX ()
{
	byte	c;

	c = getc (inf);

	if (!isxdigit(c))
		ERR_EXIT ("Invalid HEX character");
	if (c > '9')
		return (c-7-'0');
	return (c - '0');
}

byte SCAN2HEX ()
{
	unsigned	value;

	value = SCAN1HEX ();
	value = value << 4;
	value += SCAN1HEX ();

	checksum += value;

	return value;
}

SCAN4HEX ()
{
	unsigned	value;

	value = SCAN2HEX ();
	value = value << 8;
	value += SCAN2HEX ();
	return value;
}

ERR_EXIT (mess)
	char	*mess;
{
	printf ("ERROR - ");
	printf ("%s\n",mess);
	exit ();
}

SCAN_SEG ()
{
	unsigned	i;
	unsigned	count;
	unsigned	addr;
	unsigned	type;
	byte		check;
	unsigned	exec_CS;
	unsigned	exec_IP;
/*	unsigned	segment;	*/
	byte		data;


	SCAN_COLON ();
	count = SCAN2HEX ();
	addr = SCAN4HEX ();
	type = SCAN2HEX ();

	switch (type)
	{
	case 0:			/* DATA RECORD (record type 0) */
		for (i=0; i<count; i++)
			data = SCAN2HEX ();
		check = SCAN2HEX ();
		if (checksum != 0)
			printf ("Checksum error\n");
		break;
	case 1:			/* END OF FILE (record type 1) */
		check = SCAN2HEX ();
		break;
	case 2:			/* EXTENDED ADDRESS (record type 2) */
		segment = SCAN4HEX ();
		check = SCAN2HEX ();
		if ((segment < MIN_SEG) && (segment >= 0x1280))
			MIN_SEG = segment;
		if ((segment > MAX_SEG) && (segment != 0xFFFF))
			MAX_SEG = segment;
		break;
	case 3:			/* START ADDRESS (record type 3) */
		exec_CS = SCAN4HEX ();
		exec_IP = SCAN4HEX ();
		check = SCAN2HEX ();
		break;
	default:		/* UNKNOWN RECORD TYPE */
		break;
	}
	return type;
}

SCAN_RECORD ()
{
	unsigned	i;
	unsigned	count;
	unsigned	addr;
	unsigned	type;
	byte		check;
	unsigned	exec_CS;
	unsigned	exec_IP;
/*	unsigned	segment;	*/
	byte		data;

	unsigned long	abs_addr;
/*	unsigned	seg_addr;	*/
	unsigned	off_addr;

	SCAN_COLON ();
	count = SCAN2HEX ();
	addr = SCAN4HEX ();
	type = SCAN2HEX ();
#ifdef	debug
	printf ("Type %u",type);
	printf (" Count %4u",count);
	printf (" Address %04X",addr);
	printf (" Abs %04X\n",((segment - MIN_SEG) * 16) + addr);
#endif
	switch (type)
	{
	case 0:			/* DATA RECORD (record type 0) */
		if ((segment == 0xFFFF) || (segment < 0x1280))
		{
#ifdef	debug
			printf ("***** Segment ignored *****\n");
#endif
			for (i=0; i<count; i++)
				data = SCAN2HEX ();
			check = SCAN2HEX ();
			if (checksum != 0)
				printf ("Checksum error\n");
		}
		else
		{
			abs_addr = ((segment - MIN_SEG) * 16) + addr;
#ifdef	exdebug
			printf ("SEEK TO %5X\n",abs_addr);
#endif
#ifdef	debug
			printf ("%4X:%4X ",segment,addr);
#endif
#ifdef	NEW
			if (fseek (outf, 0l, 2))
				ERR_EXIT ("Seek error");
			length = ftell(outf);
			if (abs_addr > length)
			{
				for (i=0; i<(abs_addr-length); i++)
					putc (0, outf);
			}
#endif
			if (fseek (outf, abs_addr, 0))
				ERR_EXIT ("Seek error");
#ifdef	exdebug
			printf ("\n");
#endif
			for (i=0; i<count; i++)
			{
				data = SCAN2HEX ();
#ifdef	debug
				printf ("%2X ",data);
#endif
				putc (data, outf);
			}
#ifdef	debug
			printf ("\n");
#endif
			check = SCAN2HEX ();
			if (checksum != 0)
				printf ("Checksum error\n");
		}
		break;
	case 1:			/* END OF FILE (record type 1) */
		check = SCAN2HEX ();
		break;
	case 2:			/* EXTENDED ADDRESS (record type 2) */
		segment = SCAN4HEX ();
		check = SCAN2HEX ();
#ifdef	debug
		printf ("Segment %04X\n",segment);
#endif
		break;
	case 3:			/* START ADDRESS (record type 3) */
		exec_CS = SCAN4HEX ();
		exec_IP = SCAN4HEX ();
		check = SCAN2HEX ();
		printf ("Execution address IP %04X:%04X\n",exec_CS,exec_IP);
		break;
	default:		/* UNKNOWN RECORD TYPE */
		break;
	}
	return type;
}
