
/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.001        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: hexcnv.c
 * Program name: hexcnv
 * Source of file: Ron Wellstead.
 * Purpose: prepares ROM images from a hex file.
 * Changes: <who what when & why major changes have been made>      
 ********************************************************************/

/*
 *
 *	@(#) hexcnv.c 1.1 87/07/27 
 *
 */
#include	<stdio.h>
#include	<malloc.h>
#include	<string.h>
#include	<stdlib.h>

#define	TRUE	1
#define	FALSE	0

char what[]="@(#) hexcnv VR 1.0.0 4 Jun 86";

char buffer[128];
unsigned char *hi_buf,*lo_buf;
unsigned hi_addr=0;
unsigned lo_addr=0;
unsigned segment=0xf000;
unsigned map_lo=0;
unsigned map_hi=0;
unsigned map_lim=0;
unsigned dest_seg=0;
unsigned offset=0;

FILE *in_file,*out_file;

main(argc,argv)
int argc;
char **argv;
{
	char *ptr;
	int i;

	if (((hi_buf=malloc(32768))==NULL)||((lo_buf=malloc(32768))==NULL)) {
		printf("Insufficient memory to load ROM image!\n");
		exit();
	}
	if ((argc<2)||(argc>4)) {
		printf("Usage: hexcnv filename[.hex] [switches]\n");
		printf("\tValid switches :\n");
		printf("\t/c:seg:offs\t- checksum from seg:offs to seg:2\n");
		printf("\t\tstore at seg:0\n");
		printf("\t/m:seg1:limit:seg2\t- map seg1 thru limit to seg2\n");
		exit();
	}
	strcpy(buffer,argv[1]);
	if (strchr(buffer,'.')==NULL) strcat(buffer,".hex");
	if ((in_file=fopen(buffer,"r"))==NULL) {
		printf("Unable to open file %s\n",buffer);
		exit();
	}
	printf("Reading data from %s\n",buffer);
	clr_buf();
	if (argc>2) {
		for (i=2;i<argc;i++) {
			if (sscanf(argv[i],"/m:%4x:%4x:%4x",&map_lo,&map_lim,&dest_seg)==3) {
				map_hi=map_lo+(map_lim>>4);
				break;
			}
		}
	}
	load_image();
	fclose(in_file);
	if (argc>2) {
		for (i=2;i<argc;i++) {
			if (sscanf(argv[i],"/c:%4x:%4x",&segment,&offset)==2) {
				printf("Checksum = %04X\n",build_check());
				break;
			}
		}
	}
	strcpy(buffer,argv[1]);
	if ((ptr=strchr(buffer,'.'))!=NULL) *ptr='\0';
	strcat(buffer,".hx0");
	if ((out_file=fopen(buffer,"w"))==NULL) {
		printf("Unable to create file %s\n",buffer);
		exit();
	}
	printf("Creating %s\n",buffer);
	flush_buf(lo_buf);
	fprintf(out_file,":00000001FF\n");
	fclose(out_file);
	free(lo_buf);
	strcpy(buffer,argv[1]);
	if ((ptr=strchr(buffer,'.'))!=NULL) *ptr='\0';
	strcat(buffer,".hx1");
	if ((out_file=fopen(buffer,"w"))==NULL) {
		printf("Unable to create file %s\n",buffer);
		exit();
	}
	printf("Creating %s\n",buffer);
	flush_buf(hi_buf);
	fprintf(out_file,":00000001FF\n");
	fclose(out_file);
	free(hi_buf);
	printf("Conversion complete\n");
}

clr_buf()
{
	char *hi_ptr,*lo_ptr;
	unsigned count;

	hi_ptr=hi_buf;
	lo_ptr=lo_buf;
	for (count=0;count<32769;count++) {
		*hi_ptr++ = 0xff;
		*lo_ptr++ = 0xff;
	}
}

load_image()
{
	char ch;

	while (fgets(buffer,128,in_file)!=NULL) {
		convert_rec();
	}
	printf("\n");
}

convert_rec()
{
	char *ptr;
	unsigned char ch;
	unsigned offs;
	int j,i,type,len;

	for (ptr=buffer;*ptr!=':';ptr++) if (*ptr=='\0') return;
	ptr++;
	sscanf(ptr,"%2x",&len);
	ptr+=2;
	sscanf(ptr,"%4x",&offset);
	ptr+=4;
	sscanf(ptr,"%2x",&type);
	ptr+=2;
	if (type==0) {
		if ((segment>=map_lo)&&(segment<=map_hi)) {
			if ((segment+(offset>>4))<=map_hi)
				segment=dest_seg+(segment-map_lo);
		}
		if (segment>=0xf000) {
			offs = offset;
			offs += (segment<<4);
			printf("\rLoad address = %04X:%04X (%04X)",segment,offset,offs);
			for (j=0;j<len;j++) {
				sscanf(ptr,"%2x",&i);
				ptr+=2;
				ch = i & 0xff;
				if (offs&1) hi_buf[offs>>1]=ch;
				else lo_buf[offs>>1]=ch;
				offs++;
			}
		}
	 } else if (type==2) {
		sscanf(ptr,"%4x",&segment);
	}
}

flush_buf(buf)
char *buf;
{
	char *ptr;
	unsigned i,check;

	ptr=buf;
	for (i=0;i<32768;i++) {
		if (!(i%32)) {
			printf ("\rAddress = %04X ",i);
			fprintf(out_file,":20");
			check = 0x20;
			out_hex((i>>8),&check);
			out_hex(i,&check);
			out_hex(0,&check);
		}
		out_hex(*ptr++,&check);
		if ((i%32)==31) {
			check = 0 - check;
			check &= 0xff;
			fprintf(out_file,"%02X\n",check);
		}
	}
	printf("\r");
}

out_hex(ch,check_sum)
unsigned char ch;
unsigned *check_sum;
{
	fprintf(out_file,"%02X",(ch & 0xff));
	*check_sum += (ch & 0xff);
	*check_sum &= 0xff;
	return ch;
}

build_check()
{
	unsigned check=0;
	unsigned limit,offs;

	if (segment>=0xf000) {
		offs=(segment<<4);
		limit=offs>>1;
		offs+=offset;
		for (offs>>=1;offs>limit;offs--)
			check+=((hi_buf[offs]<<8)+lo_buf[offs]);
		hi_buf[limit]=(check>>8);
		lo_buf[limit]=(check & 0xff);
	}
	return check;
}
